// Treiber fr Ansteuerung Timboy mit RS485 oder RS232
// Sprache: MS Visual C++ 5.0
// 16.5.2000 Keller Version 1.0
//

#include <winsock.h>
#include "DFLog.h"						

#define StdBufSize 4096


#define ComPortBaudRate_Min            9600

#define ClearComReadPort_TimeOut_RS232 (time_t)  0 // 0 means use StandardTimeOut from PortOpen
#define ClearComReadPort_TimeOut_Radio (time_t) 10

//#define WriteMem_ReadRetry_Default      8
//#define  ReadMem_ReadRetry_Default      10
// ZINN
#define WriteMem_ReadRetry_Default      50
#define WriteMem_Retry_Reinit			8 // nach x erfolglosen versuchen Schnittstelle aufrumen und kurz warten
#define  ReadMem_ReadRetry_Default      40
#define ZK 1 // Original: 1


#define COM_LOG_FILENAME "TbCom_%i.log"



#define MAX_EEP_MEM		0x1000		// Buffer fr EEP-Daten Timeboy
#define MAX_FLASH_MEM	0x110000	// Buffer fr Falsh-Daten Timeboy
#define MAX_LIST_DAT	0x100000
#define MB_KOM_END		0x40010000	// Lesen von EEPROM mit spezieller Adresse
#define MB_KOM_START	0x40020000	
#define MB_KOM_AUS		0x40040000
#define MB_MESSAGE		0x40100000
#define MB_GLOB_VAR		0x40200000	// Glob.-Var. in Timeboy setzen
#define MB_FUNK_AUS		0x40800000	// Schaltet Funk im TB aus
#define MAX_MB	300					// max. MOD-Bus Datenpaket
#define MB_DOCK_LED		0x80001000	// Rote LED ein/aus
#define MB_READ_ONLINE	0x48000000	// Offset fr Read einen Datensatz Online bzw. fr lschen ersten Datensatz


#define MAX_NETZ_GERAETE	31	// max. Gerte am Netz (ohne PC)

// Rckgabewerte von GetDatOnline und GetDatensatzAndClr
#define RET_OK		1	// Funktion erfolgreich ausgefhrt
#define RET_NO_TB	0	// Timboy antwortet nicht
#define RET_ERROR	2	// Fehler Kommunikation

// Fehler
#define ERR_NO			0	// kein Fehler
#define ERR_CRC			1	// CRC von Datensatz
#define ERR_OPEN		2	// OPEN einer Datei gescheitert
#define ERR_FORMAT		3	// Z.B. falsches Format einer Setup-Datei
#define ERR_NO_TB		4	// Timebox antwortet nicht
#define ERR_KOM			5	// Fehler bei Kommunikation mit Timeboy
#define ERR_NO_COM		6	// Schnittstelle nicht geffnet
#define ERR_NO_LIST		7	// keine Listen vorhanden
#define ERR_NO_DOCK		8	// Dockingstation antwortet nicht
#define ERR_PASSWORT	9	// falsches Passwort
#define ERR_BAUDRATE	10	// falsche Baudrate
#define ERR_LISTEN_LOESCHEN 11 // Fehler beim lschen der Listendaten.
#define ERR_KONFIG_SIZE	12	// Konfigdaten zu gro fr die Firmware 
#define ERR_ALT_SETUP	13	// Es wird versucht eine alte Setupdatei in ein Gert mit Firmware >= 3.2 zu laden

#define ERR_ABORT		20	// Abbruch durch Callback Funktion
#define ERR_CHANNEL		21  // Kanalobjekt konnte nicht erstellt oder ermittelt werden
#define ERR_FUNCTION_PARAMETER       22  // Funktionsparameter (ungueltig) auserhalb seines Gueltigkeitsbereiches

extern unsigned int ComThread(LPVOID pParam);

//#define PC_SIMULATION	// Fr Einbindung der mit der Firmware gemeinsam verwendeten Dateien
//#include "tb.h"
#include "Setup.h"

#define MAX_WBUF	0x100
#define MAX_RBUF	0x100
// Modbus
#define MB_ADD_OFFLINE 0xFE		// Modbus Adresse Offline
//	Befehle
#define MB_READ		0x21		// bis zu 64 Byte lesen
#define MB_WRITE	0x22		// bis zu 64 Byte schreiben

// Adressen
#define MB_START_EEP	0x40000000	// Satrtadresse EEP-ROM
#define MB_CLR_BLOCK	0x47000000	// Offset fr Block lschen im Flash
#define MB_SET_TIME		0x43000000	// Offset fr Uhr stellen
#define MB_GET_TIME		0x44000000	// Offset fr Uhr lesen
#define MB_GET_VERSION	0x42000000	// Offset fr Version lesen

#define MB_DOCK_BUS_NUM	0x80000000	// Busnummer

#define MB_LISTE_LADEN_INIT 0x49030000	// Listen laden einzeln vorbereiten
#define MB_LISTE_LADEN_END 0x49040000	// Listen laden einzeln abschliesen
#define MB_LISTE_EINZELN 0x4A000000	// Listen laden einzeln Daten

#define MAX_GLOB_STR	41


//*****************************************
// Neue Konstante

#define MAX_DOCK_MODEM_STR	128
#define MAX_VERSION_STR		21

//    Dock.
//Hex 				Beschreibung		Wr	Datentyp	Werte	Default
//--------------------------------------------------------------------------------
//0x 80 00 00 00	Busadresse			ja	int			0 .. 255	254
#define MB_DOCK_BUS_NUM		0x80000000

//0x 80 00 00 02	Baudrate zum PC		ja	int			2400,
//														4800, 
//														9600,
//														19200,
//														38400		9600
#define MB_DOCK_BAUD_PC		0x80000002

//0x 80 00 00 04	Baudrate zum Tb		ja	int			2400, 
//														4800, 
//														9600,
//														19200,
//														38400		9600
#define MB_DOCK_BAUD_TB		0x80000004

//0x 80 00 00 06	Timeout "Kein Gert"
//					(Wert mal 10 msec)	ja	int			0 .. 250	20
#define MB_DOCK_TIMEOUT		0x80000006

//0x 80 00 00 08	Modemfunktionen 
//					aktivieren			ja	int			0, 1		0
#define MB_DOCK_MODEM_STATE 0x80000008 

//0x 80 00 10 00	LED schalten		ja	char		0, 1		0
#define MB_DOCK_LED			0x80001000	// Rote LED ein/aus

//0x 80 00 20 00	String fr Modem	ja	char		0 .. 255	--
#define MB_DOCK_MODEM_STR	0x80002000

//0x 80 00 FF FF	Softwareversion 	--	int			0x01nn		0x0100
#define MB_DOCK_VERSION		0x8000FFFF


// Funk
#define LEN_PREAMBLE            30

#define ZEICHEN_PREAMBLE        0x55
#define ZEICHEN_SYNCHRO         0xFF
#define ZEICHEN_START           0xAA

#define MAX_FUNK_BUFFER			0x300

#define FUNK_BAUD	9600

#define LLM_WARTESEND_PAKET     42										// (42)
#define LLM_PREAMBLES_START     LLM_WARTESEND_PAKET     + 1             // (43)
#define LLM_PREAMBLES_END       LLM_PREAMBLES_START     + LEN_PREAMBLE  // (73)
#define LLM_SEND_SYNCHRO1       LLM_PREAMBLES_END       + 1             // (74)
#define LLM_SEND_SYNCHRO2       LLM_SEND_SYNCHRO1       + 1             // (75)
#define LLM_SEND_START          LLM_SEND_SYNCHRO2       + 1             // (76)
#define LLM_SEND_PAKET          LLM_SEND_START          + 1             // (77)
#define LLM_WARTE_PREAMBLE      LLM_SEND_PAKET          + 1             // (78)
#define LLM_PREAMBLE_W1         LLM_WARTE_PREAMBLE      + 1             // (79)
#define LLM_WARTE_START         LLM_PREAMBLE_W1         + 1             // (80)
#define LLM_WARTE_EMPF          LLM_WARTE_START         + 1             // (81)
#define LLM_EMPF_PAKET          LLM_WARTE_EMPF          + 1             // (82)

#define ANZ_ERRORZEICHEN    3       /* max. 3 falsche Zeichen zwischen Prambel und Start */
#define MAX_WARTEN_START	1400	/* nach Anfrage mu Slave nach dieser Zeichenzahl Startzeichen geschickt haben */

union U_intbyte
{
   int   IntWert     ;
   unsigned short  WortWert    ;
   unsigned char  ByteWert[2];
   unsigned char EditWert[2];
};

class CCom: public CDFLog
{
public:
	CCom(int Num);
	virtual ~CCom();

	enum PortType { ComUndef, ComSerial, ComSocket };

	HANDLE GetComPort();
	BOOL SetComPort(HANDLE ComPort, BOOL PortIsSocket, int BaudRate, time_t TimeOut);

	int  ComOpenSerial(char *ComPort, int BaudRate, time_t TimeOut);
	int  ComOpenSocket(char *RemoteIP, int PortNo, time_t TimeOut);
	void ComClose();

	// Helper
	int IsOpen();

	// Log
	void SetLogFileName(const unsigned char *LogFileName);
	void SetLogOn() {LogIsOn = TRUE;}
	void SetLogOff() {LogIsOn = FALSE;}

	// Funk
	void SetFunkOn();
	void SetFunkOff(); 
	void SetZigBeeOn(); 
	void SetZigBeeOff();

	// Timeboy 2
	void NurTbIIOn()  {NurTbII = TRUE;}	//  NTB2
	void NurTbIIOff() {NurTbII = FALSE;} // NTB2

	// New
	void SetComCheckBeginEndOn();
	void SetComCheckBeginEndOff();

	int  TbEndCom(unsigned char BusNum, int *Error);		// Schickt Befehl zum beenden der Kom. an TB
	int  TbBeginCom(unsigned char BusNum, int *Error);	// Schickt Befehl zum Start einer kritischen Kom. an TB
	int  TbCheckDevice(unsigned char BusNum,int *Error, int DockingPollRetry, int DevicePollRetry);	
	int  TbCheckDeviceDefault(unsigned char BusNum, int *Error, int Fast=FALSE);
	void TbGetDevicePollRetry(int *DockingPollRetry, int *DevicePollRetry);	

	// Routinen zum Einstellen der Dockingstation
	int GetDockModemState(unsigned char BusNum, int *Error, int *State);
	int SetDockModemState(unsigned char BusNum, int *Error, int State);
	int GetDockModemString(unsigned char BusNum, int *Error, char *String);
	int SetDockModemString(unsigned char BusNum, int *Error, char *String);
	int GetDockTimeOut(unsigned char BusNum, int *Error, int *TimeOut);
	int SetDockTimeOut(unsigned char BusNum, int *Error, int TimeOut);
	int SetDockBaudRate(unsigned char BusNum, int *Error, int BaudRate);
	int SetDockBusNum(unsigned char BusNum, int *Error, unsigned char NewBusNum);


	volatile int Version;
	volatile int ComNum;
	unsigned char EepMem[MAX_EEP_MEM];
	unsigned char FlashMem[MAX_FLASH_MEM];
	int		LSize;	// Gre der Listen
	int		LSizeV,LSizeF;
	volatile int BusNum;
	CString Passwort;
	int CntFunk;	// Wartezeit auf Paket in Byte bei Funk
	int SlListStart[MAX_DATENSATZ_BESCHREIBUNG];
	int SlListSize[MAX_DATENSATZ_BESCHREIBUNG];

	// Datensatzbeschreibung
	volatile unsigned long StartDatensatzB,EndDatensatzB;
	TDatensatzBeschreibung DatensatzBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld DatensatzFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD+10];
	int MaxDatB;

	// Listenbeschreibung
	TDatensatzBeschreibung ListenBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	int	ListenZeileSize[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld ListenFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD+10];
	int MaxListB;
	ULONG FlagListenVF;
	int Abbruch;
	int ListMemSizeNum;
	int ListMemSizeNumV;
	unsigned char ListDatBuf[MAX_LIST_DAT];	// Buffer fr zurckgelesene Liste
	int ListDatNr;	// Nummer der Liste im Buffer
	int ListDatCnt;	// Anzahl Listenzeilen im Buffer
	int ListDatFeldCnt;	// Anzahl Felder
	int ListDatReady;	// Listendaten sind vorhanden

	void ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val);
	void UIntToBinMSBFirst(unsigned char *Buf,unsigned int Val);
	ULONG BinToULongMSBFirst(unsigned char *Dat);
	int CheckMBPaket(unsigned char *buf, int cnt);
	int BinToUIntMSBFirst(unsigned char *Buf);
	int WriteMem(unsigned long Add,int Cnt,unsigned char *Buf, int WriteRetry = WriteMem_ReadRetry_Default);
	int ReadMem(unsigned long Add,int Cnt,unsigned char *Buf, int ReadRetry = ReadMem_ReadRetry_Default);

	int GetDatenOnline(int *Error, int *DatLen, int *DatBeschreibungNum, unsigned char *DatBuf);
	int ClrDatenOnline(int *Error);
	void CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned long Add,int Cnt);
	int GetDatensatz(unsigned char *DatS,unsigned char *DatBuf,ULONG *pRd,ULONG pEnd,int *Err);
	void GetFeld(CString *Feld,unsigned char *Dat,void *FeldBeschreibung,int *Pos);
	int ClrDat(int *Error);
	int SetSetup(int *Error, int Size, int Version2);
	int GetDaten(int *Error);
	int GetDatensatzbeschreibung(int *Error, int ListenB=FALSE, int ListNr=-1);
	int ScanDaten(int *Error);
	int GetDatCnt();
	int GetDatBCnt(int ListenB=FALSE);
	int GetDatLen(int N);
	int GetDatBeschreibungNum(int N);
	int CpyDat(int N, unsigned char *Buf);
	int SetListen(int *Error);
	int SetListenVF(int *Error,int NurVarListen);
	int SetDateTime(unsigned char BusNum,int *Error, unsigned char *Buf);
	int GetDateTime(unsigned char BusNum,int *Error, unsigned char *Buf);
	int GetPassword(unsigned char BusNum,int *Error, unsigned char *Password);
	void ChangeBusNum(int Num);
	void SetPasswort(CString Pass);
	void ClrListen();
	int TBAus(unsigned char BusNum,int *Error);	
	int TbSwitchTimeboyFunkOff(unsigned char BusNum, int *Error);	// Schickt Befehl zum Ausschalten des Funk im Timeboy
	int TBMessage(unsigned char BusNum, int *Error, unsigned char *Mess, BOOL MessageIsFormatted=FALSE, BOOL SendLineByLine=FALSE);
	int TBSetGlobVar(unsigned char BusNum,unsigned char *GlobVarS1,unsigned char *GlobVarS2,
					unsigned char *GlobVarS3,unsigned char *GlobVarS4,
					unsigned int GlobVarI1,unsigned int GlobVarI2,unsigned int GlobVarI3,
					unsigned int GlobVarI4,int *Error);
	int TBGetGlobVar(unsigned char BusNum,unsigned char *GlobVarS1,unsigned char *GlobVarS2,
					unsigned char *GlobVarS3,unsigned char *GlobVarS4,
					unsigned int *GlobVarI1,unsigned int *GlobVarI2,unsigned int *GlobVarI3,
					unsigned int *GlobVarI4,int *Error);
	int IsClrListen,IsClrListenVF;
	int Speicher,SpeicherV;
	int ListStartV,ListStartF;
	unsigned char PMaske[50*sizeof(long)+10];
	int GetDatBFeldCnt(int DatBeschreibungNum);
	int GetDatBDatensatz(int DatBeschreibungNum,unsigned char *Name,int *Felder,int *Var=NULL,int ListB=FALSE);
	int GetDatBFeld(int DatBeschreibungNum,int Feld,unsigned char *Name,int *Typ,int *Len,int ListB=FALSE);
	void SetCBF(int (*pF)()) {CBFunction = pF;}
	int GetSerienNum(unsigned char BusNum,int *Error, int *Seriennummer);
	int GetListSizeNum() {return ListMemSizeNum;}
	int GetListSizeNumV() {return ListMemSizeNumV;}
	int SetListeSingle(int ListNum, int *Error);

	void ComLog(const char *Format, ...);

	int GetVersion(unsigned char BusNum, int *Error, char *Version);
	int ClrListenTb(unsigned char BusNum,int *Error);

	int ComFunk;
	int ComZigBee;
	int NurTbII;
	int ListDGet(int Zeile,int Feld,unsigned char *Buf);
	int ListDCnt(int *Cnt);
	int ListDFeldCnt(int *FCnt);
protected:
	int  InstanzNummer;

	int  LogCnt;
	BOOL LogIsOn;
	CString LogDateiName;

	PortType ComPortType;  // Typ der Schnittstelle
	int  ComPortTimeOut;
	int  ComPortBaudRate;
	BOOL ComCheckBeginEnd;
	int  ComDockingPollRetry;
	int  ComDevicePollRetry;
	// Fehlerkorrektur
	int DatensatzLen[MAX_DATENSATZ_BESCHREIBUNG];

	HANDLE hcom;	       // Handle serielle   Schnittstelle
	SOCKET Socket;         // Handle TCP_Socket Schnittstelle

	int Fehler23Cnt;
	int FlagAlteTB;

	int  ClearComReadPort(time_t TimeOut, int ReadRetry=1);
	int SetComTimeout(int TimeOutMS); // alte Version nur fr Funk
	int  SetSerialComTimeOut(HANDLE ComPort, time_t TimeOut);
	int  SetSerialComState(HANDLE ComPort, int BaudRate, time_t TimeOut);
	int  SerialOpen(char *ComPort, int BaudRate, time_t TimeOut);
	int  SocketOpen(char *RemoteIP, int PortNo, time_t TimeOut);
	void SocketClose(void);
	int  SocketRead(char *InputBuf, int InputLen, DWORD *ReadLen);
	int  SocketWrite(const char *OutputBuf, int OutputLen, DWORD *WriteLen);
	int ChkDatensatz(unsigned char *DatBuf,ULONG *pRd,ULONG pEnd,int *Err);
	int FehlerKorrektur(unsigned char *DatBuf,ULONG *pRd,ULONG pEnd,int *Err,int DatensatzLen[],int MaxDatB);

public:
	int  ComRead(void *Buf, int Len, DWORD *ReadLen);
	int  ComWrite(void *Buf, int Len, DWORD *WriteLen);

protected:
	ULONG StartDat,WritePt;	// Daten in FlashMem Startadd. und Ende
	int DatPt[80000];	// Startadressen der Datenstze in "FlashMem"
								// negative Werte sind Fehler
	int	DatCnt;		// Anzahl gelesene Datenstze
	int (*CBFunction)();

	void ASCII_to_diphase(unsigned char Zeichen, unsigned char *Pointer_String);
	unsigned char diphase_to_ASCII (unsigned char *Pointer_String);
	int MkTxdBufferFunk(unsigned char *sbuf,int cnt,unsigned char *FunkBuf);
	int CommandFunk(unsigned char *sbuf,int cnt,unsigned char *ebuf,int *ret);

	int CommandRS232(unsigned char *sbuf,int cnt,unsigned char *ebuf,int *ret);
	int Command(unsigned char *sbuf,int cnt,unsigned char *ebuf,int *ret);

	void MkMBPaket(unsigned char *buf,int cnt);	// CRC berechnen
	void GetSetup();
	int CatchTb(int PollRetry);
	int CatchDock(int PollRetry);
	int ReadXByte (unsigned long Add,int Cnt,unsigned char *Buf, int *Error);
	int WriteXByte (unsigned long Add,int Cnt,unsigned char *Buf, int *Error);
	int CheckPasswort();
	int ChkPasswortString(unsigned char *buf);
	int ExeCallBack();
	int ReadFunk(HANDLE hcom,LPVOID buf,DWORD cnt,LPDWORD nb,void *p);
	int WriteFunk(HANDLE hcom,LPVOID buf,DWORD cnt,LPDWORD nb,void *p);
};
