/**
 * @file  DFFileIni.cpp
 * @brief Implementierung der Klasse CDFFileIni.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFFileIni.h"

CDFFileIni::CDFFileIni()
{
}

CDFFileIni::~CDFFileIni()
{
}

int CDFFileIni::ExistingFile(char *fileName) 
{
	struct stat st;

	if (stat(fileName, &st) == 0) {
		if (st.st_size > 0) { // Datei mit Inhalt
			return 1;
		}
	}

	return 0;
}

int CDFFileIni::Scan(char *section, char *key, char *value, unsigned long flags, char *fileName)
{
	FILE *fpIn, *fpOut;
	char fileName1[260];

	char buf[0x1000]; // 4k Puffer
	char *pBuf = buf;

	// Zustaende der Suche
	enum { 
		ENTRY, COMMEND, SECTION, KEY, VALUE, EXIT 
	} majorState, majorJump;
	// Unterzustaende der Suche
	enum {
		FIRST, SECOND, THIRD, FOURTH
	} minorState, minorJump;

	// Allgemeine Verarbeitungsvariable
	int c,
		lineNum = 0,
		search = 1, res = 0;

	// Gefundene aufbereitete Namen
	char sectionName[MAX_SECTION_NAME];
	int iSection, lineNumSection, lineNumSectionLast;

	char keyName[MAX_KEY_NAME];
	int iKey, lineNumKey, lineNumKeyLast;

	char valueName[MAX_VALUE_NAME];
	int iValue;
	int quote, unquote, esc;

	char *commendName;
	int iCommend;

	// Eingangsdatei oeffnen
	if ((fpIn = fopen(fileName, "r")) == (FILE*)0) {
	
		return res;
	}

	// Werte abaendern, erzeugen?
	if (flags & SF_WRITE) {
		strcpy(fileName1, fileName);
		strcat(fileName1, "_");
		// Ausgangsdatei oeffnen
		if ((fpOut = fopen(fileName1, "w")) == (FILE*)0) {
		
			fclose(fpIn);
			return res;
		}
	} else if (flags & SF_CREATE) {

		// Flag ausblenden da Unsinn!
		flags &= ~SF_CREATE;
	}

	// Basiszustand fuer den Hautpzustand
	majorState = ENTRY;
	majorJump  = ENTRY;

	// Basiszustand fuer den Unterzustand
	minorState = FIRST;
	minorJump  = FIRST;

	// Die gefundene Sektions- sowie Schluesselzeile wird gehalten
	lineNumSection = -1;
	lineNumKey     = -1;

	// Die letzte gefundene Sektions- sowie Schluesselzeile wird gehalten
	lineNumSectionLast = -1;
	lineNumKeyLast     = -1;

	// Zeichenweise einlesen
	while (search && !feof(fpIn) && !ferror(fpIn)) {

		if ((c = fgetc(fpIn)) == EOF) {

			// Restverarbeitung je nach aktuellem Zustand fortsetzen
			switch (majorState) {
			case ENTRY:
				switch (minorState) {

				case FIRST:
					break;

				default: // EOF in Verarbeitungsprozess
					search = 0;
					res = 0;
					continue;
				}

				// Wiedereinsprung
				majorJump  = ENTRY;
				minorJump  = FIRST;

				// Auf Endzustand wechseln
				majorState = EXIT;

				// Ausgabepuffer
				*pBuf++ = 0;
				*pBuf = 0; // Nullterminie
				break;

			case KEY: 
				switch (minorState) {

				case FIRST:
					minorState = SECOND;
					break;

				default: // EOF in Verarbeitungsprozess
					search = 0;
					res = 0;
					continue;
				}

				// Wiedereinsprung
				majorJump  = ENTRY;
				minorJump  = FIRST;

				// Auf Endzustand wechseln
				majorState = EXIT;

				// Ausgabepuffer
				*pBuf++ = 0;
				*pBuf = 0; // Nullterminie
				break;

			case SECTION: // EOF hinter Sektion
			case COMMEND: // EOF hinter Kommentar
			case VALUE:   // EOF hinter Wert
				
				ungetc(c, fpIn); // EOF zurueck
				clearerr(fpIn);  // EOF loeschen

				c = '\n'; // Fehlendes Linefeed einfuegen
				*pBuf++ = (char)c; 
				*pBuf = 0;
				break;
			
			case EXIT:

				break;

			default: // Hautpzustand unbekannt
				search = 0;
				res = 0;
				continue;
			}

		} else {

			// Ausgabepuffer
			*pBuf++ = (char)c;
			*pBuf = 0; // Nullterminie
		}

		// Hauptzustand auswerten!
		switch (majorState) {

		case ENTRY: {
			switch (minorState) {
			case FIRST:
				if (c == '\n') { 	
					lineNum++; 
				} else if (isspace(c)) {
					
				} else if (c == ';') { 

					// Zustand wechseln
					majorJump  = ENTRY;
					majorState = COMMEND;
					minorState = FIRST;
					
				} else if (c == '[') { 

					// Zustand wechseln
					majorJump  = ENTRY;
					majorState = SECTION; 
					minorState = FIRST;

				}  
				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}

			break;}

		case COMMEND: {

			switch (minorState) {
			case FIRST:
				iCommend = 0;

				// Startzeichen setzten
				commendName = (char*)&pBuf[-2];

				// Zustand wechseln
				minorState = SECOND;

			case SECOND:
				if (c == '\n') { 

					// Token ausgeben
					//TRACE("COMMEND( %03i ): %s\n", lineNum, commendName);

					lineNum++;

					// Gehe hin wo du her kommst!
					majorState = majorJump;
					minorState = minorJump;

					break;
				}

				// Zeichen zur Kommentarzeile aufnehmen
				iCommend++;
				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}

			break;}


		case SECTION: {

			switch (minorState) {
			case FIRST:

				// Zeile der letzten gefundenen Sektion halten
				lineNumSectionLast = lineNum;

				iSection = 0; 
				memset(sectionName, 0, MAX_SECTION_NAME);

				// Zustand wechseln
				minorState = SECOND;

			case SECOND:
				if (c == ']') {

					// Mit nullterminie!
					sectionName[iSection] = 0;

					// Ist benoetigte Sektion?
					if (strcmp(section, sectionName) == 0) {

						// Zeile der gefundenen Sektion halten
						lineNumSection = lineNum;

						// Zustand wechseln
						majorJump  = KEY;
						minorJump  = FIRST;

						minorState = THIRD; // auf Zeilenumbruch warten 

					} else {

						// Zustand wechseln
						majorJump  = ENTRY;
						minorJump  = FIRST;

						minorState = THIRD;
					} 

					break;

				} else if (c == '\n') { 
					// Kein Zeilenumbruch in Sektionsbezeichner
					search = 0; 
					res    = 0; 
					continue;
				} 

				// Zeichen zum Sektionsnamen aufnehmen
				sectionName[iSection++] = (char)c;
				break;

			case THIRD:
				if (c == '\n') {

					// Token ausgeben
					//TRACE("SECTION( %03i ): %s\n", lineNum, sectionName);

					lineNum++;

					// Zustand wechseln
					majorState = majorJump;
					minorState = minorJump;

					break;
				} else if (isspace(c)) {
					break;
				}

				// Fehler, undefiniertes Zeichen nach Sektionsbekanntgabe
				search = 0;
				res = 0;
				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}
			break;}

		case KEY: {
			switch (minorState) {
			case FIRST:
				if (c == '\n') { 

					lineNum++;
					break;
				} else if (isspace(c)) {
					
					break;
				} else if (c == ';') { 

					// Zustand wechseln
					majorJump  = KEY;
					minorJump  = FIRST;

					majorState = COMMEND;
					minorState = FIRST;
					break;
					
				} else if (c == '[') { 

					// Neue Sektion

					// Zustand wechseln
					majorJump  = ENTRY;
					minorJump  = FIRST;

					majorState = EXIT; // Schluessel unter dieser Sektion nicht gefunden 
					minorState = SECOND;
					break;
					
				}
				
				// Zeile des letzten gefundenen Schluessels halten
				lineNumKeyLast = lineNum;

				iKey = 0;
				memset(keyName, 0, MAX_KEY_NAME);

				// Zustand wechseln
				minorState = SECOND;

			case SECOND:
				if (c == '=') { 

					// Null terminie!
					keyName[iKey] = 0;

					// Folgende Leerzeichen von Schlusselbezeichner entfernen
					for (int j=strlen(keyName) - 1; j; j--) { // RTrim
						if (isspace(keyName[j])) {

							keyName[j] = 0;
						} else {

							break;
						}
					}

					// Ist gesuchter Schluessel
					if (strcmp(key, keyName) == 0) {

						// Zeile des gefundenen Schluessels halten
						lineNumKey = lineNum;

						// Zustand wechseln
						majorJump  = EXIT;
						minorJump  = THIRD;

						majorState = VALUE;
						minorState = SECOND;

					} else {

						iValue = 0;
						memset(valueName, 0, MAX_VALUE_NAME);

						// Zustand wechseln
						majorJump  = KEY;
						minorJump  = FIRST;

						majorState = VALUE;
						minorState = FIRST;
					}

					// Token ausgeben
					//TRACE("KEY( %03i ): %s\n", lineNum, keyName);

					break; 
				} else if (c == '\n') { 
					// Kein Zeilenumbruch in Schluesselbezeichner
					search = 0; 
					res = 0; 
					continue; 
				}

				// Schluesselnamen aufnehmen
				keyName[iKey++] = (char)c;
				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}
			break;}

		case VALUE: {
			switch (minorState) {
			case FIRST:
				if (c == '\n') {

					// Token ausgeben
					//TRACE("RECYCLE_VALUE( %03i ): %s\n", lineNum, valueName);

					lineNum++;

					// Zustand wechseln
					majorState = majorJump;
					minorState = minorJump;

					break;
				}

				valueName[iValue++] = (char)c;
				break;

			case SECOND:
				// Fuehrende Whitespace ohne LF uebergehen
				if (isspace(c) && (c != '\n')) {

					break;
				}

				iValue = 0;
				memset(valueName, 0, MAX_VALUE_NAME);

				// Zustand wechseln
				minorState = THIRD;

			case THIRD:
				if (c == '\n') { 

					// Null terminie!
					valueName[iValue] = 0;

					// Wert lesen?
					if (flags & SF_READ) {

						strcpy(value, valueName);
					} else if ((flags & SF_WRITE) && (strcmp(valueName, value) != 0)) {
						// Wert schreiben, wenn ungleich!
						char *p = pBuf - 1, *b = value;

						// Auf Gleichheitszeichen zurueckgehen
						for (; *p != '='; p--);

						p++; // Dahinter und Leerzeichen sowie Hochkomma einfuegen
						*p++ = ' ';
						*p++ = '"';

						// Wert ueber alten schreiben
						for (; *b; b++, p++) *p = *b;

						*p++ = '"'; // Hochkomma und abschliessende Linefeed einfuegen
						*p++ = '\n';
						*p = 0;

						// Aktuellen Zeigerzustand uebernehmen
						pBuf = p;
					}

					//TRACE("VALUE( %03i ): %s [%s]", lineNum, valueName, value);

					lineNum++;

					// Kontrollflags zuruecksetzten
					esc = 0;
					quote = 0;
					unquote = 0;

					// Zustand wechseln
					majorState = majorJump;
					minorState = minorJump;

					break;

				} else if (!esc && !quote && (c == '"')) {
					// einleitendes Gaenzefuesschen abfangen

					quote = 1;

				} else if (!esc && quote && (c == '"')) {
					// ausleitendes Gaenzefuesschen abfangen

					unquote = 1;

				} else if (!esc && (c == '\\')) {
					// Maskierungszeichen abfangen

					esc = 1;

				} else {

					// Zeichen nach escapezeichen durchlassen
					esc = 0;
				}

				valueName[iValue++] = (char)c;
				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}
			break;}

		case EXIT: {
			switch (minorState) {

			case FIRST: // EOF erreicht, Sektion nicht gefunden
				if ((flags & SF_WRITE) && (flags & SF_CREATE)) {
					
					// Restpuffer schreiben
					fprintf(fpOut, "%s", buf);
					pBuf = buf;
					*pBuf = 0;
					
					// Sektionsangabe mit Schluessel ausgeben
					fprintf(fpOut, "[%s]\n", section);
					fprintf(fpOut, "%s = \"%s\"\n", key, value);

					search = 0;
					res = 1;
				}
				break;

			case SECOND: // Sektion gefunden, aber Schluessel nicht
				
				if ((flags & SF_WRITE) && (flags & SF_CREATE)) {
					
					int line = lineNum, lineToSet;
					char *p = pBuf - 1;
					pBuf[1] = 0;

					// Hatte Sektion Schluessel unter sich?
					if (lineNumSection < lineNumKeyLast) {
						// Bis hinter letzten Schluessel zurueck!
						lineToSet = lineNumKeyLast;

					} else {
						// Bis hinter Sektion gehen!
						lineToSet = lineNumSection;
					}

					for (; (p > buf) && (lineToSet < line); (lineToSet < line) ? p-- : p) {
						
						if (*p == '\n') {
							line--;
						}
					}

					*p = '\0';

					// Vorpuffer schreiben
					fprintf(fpOut, "%s", buf);
					
					// Schluessel schreiben
					fprintf(fpOut, "\n%s = \"%s\"\n", key, value);
					
					// Restpuffer schreiben
					p++;
					fprintf(fpOut, "%s", p);

					pBuf = buf;
					*pBuf = 0;

					minorState = THIRD;

					// Rueckgabe fuer Erfolgreiches anlegen
					res = 1;
				}
				break;

			case THIRD:

				if (flags & SF_WRITE) {

					// Angesammelten Puffer schreiben
					fprintf(fpOut, "%s", buf);
					pBuf = buf;
				}

				break;

			default: // Fehler in Zustand
				search = 0;
				res = 0;
			}
			break;}

		default: // Fehler in Zustand
			search = 0;
			res = 0;
		}
	}

	// Dateien schliessen
	fclose(fpIn);
	if (flags & SF_WRITE) {
	
		fclose(fpOut);

		// Orginaldatei loeschen
		if (remove(fileName) == 0) {
		
			// Zwischendatei umbenennen
			if (rename(fileName1, fileName) != 0) {
				
				res = 0;
			}
		}
	}

	return res;
}

int CDFFileIni::FindSection(FILE *fp, char *section, unsigned long flags)
{
	char line[0xFF];
	int res = 0, pos;
	int search = 1;

	while (search && !feof(fp) && !ferror(fp)) {

		if (fgets(line, 0xFF, fp) != NULL) {
			
			// Kommentarzeilen uebergehen (erstes gueltiges Zeichen ist ein Semikolon)
			for (pos=0; isspace(line[pos]); pos++); 
			if (line[pos] == ';') continue;

			if (strstr(&line[pos], section) != NULL) {
				res = 1;
				search = 0;
			}
		} 
	}

	// Bei Fehler, abbruch
	if (ferror(fp)) {

		clearerr(fp);
		res = 0;
		search = 0;
	}
	
	if (search && (flags == FF_CREATE)) {
		// Erzeugen wenn nicht vorhanden
		fprintf(fp, "%s%s", section, "\n");
		fseek(fp, 0, SEEK_END);
		res = 1;
		search = 0;
	}

	return res;
}

int CDFFileIni::FindKey(FILE *fp, char *key, unsigned long flags, char *line, int length)
{
	int res = 0, pos;
	int search = 1;

	while (search && !feof(fp) && !ferror(fp)) {

		if (fgets(line, length, fp) != NULL) {
			
			// Kommentarzeilen uebergehen 
			for (pos=0; isspace(line[pos]); pos++); 
			// erstes gueltiges Zeichen ist ein Semikolon
			if (line[pos] == ';') continue;
			// Neue Sektion beendet die Suche
			if (line[pos] == '[') {

				fseek(fp, -(long)(strlen(line) + 1), SEEK_CUR);
				break;

			}

			if (strstr(&line[pos], key) != NULL) {
				res = 1;
				search = 0;
			}
		} 
	}

	// Bei Fehler, abbruch
	if (ferror(fp)) {

		clearerr(fp);
		res = 0;
		search = 0;
	}

	if (search && (flags == FF_CREATE)) {
		// Erzeugen wenn nicht vorhanden
		sprintf(line, "%s = ?\n", key);
		fprintf(fp, line);
		//fseek(fp, -(strlen(key) + strlen("\n") + 1), SEEK_CUR);
		fseek(fp, 0, SEEK_END);
		res = 1;
		search = 0;
	}

	return res;
}

unsigned int CDFFileIni::GetNum(FILE *fp, int defaultVal, char *line)
{
	char number[11];
	memset(number, 0, 11);

	char *pos = strstr(line, "=");

	if (pos) {
	
		pos++;
		for (; isspace(*pos); pos++); // Leerzeichen uebergehen
		for (int i=0; isdigit(*pos); pos++, i++) {
			number[i] = *pos; // Zahl aufnehmen
		}
		return atoi(number);
	}

	return (defaultVal < 0) ? 0 : defaultVal;
}

int CDFFileIni::GetStr(FILE *fp, char *value, char *line)
{
	unsigned char mask = 0, quote = 0, unquote = 0, end = 0;
	char *pos = strstr(line, "=");

	if (pos) {
	
		pos++;
		for (; isspace(*pos); pos++); // Leerzeichen uebergehen
		if (*pos == '"') {
			pos++;
			quote = 1; // Anfuehrungszeichen unten
		}
		int i = 0;
		for (; IsPrint(*pos); pos++, i++) {
			
			if (!mask && *pos == '\\') {
				
				mask = 1; // Maskierungszeichen erkannt
				continue;
			} else {

				// War letztes Zeichen ein Maskierungszeichen, dann dieses uebernehmen
				if (mask) { 
				
					mask = 0;
					i--;
				// Kein Maskierungszeichen, dann vielleicht abschliessende quote
				} else if (quote && *pos == '"') {

					unquote = 1;
					break; // nach abschliessender quote abbruch!
				}

				value[i] = *pos; // Zeichenkette aufnehmen
			}
		}
		value[i] = 0; // Nullterminie
		return 1;
	}

	return 0;
}

int CDFFileIni::SetStr(FILE *fp, char *value, char *line)
{
	unsigned char mask = 0, quote = 0, unquote = 0, end = 0;
	char *pos = strstr(line, "=");

	if (pos) {
		
		int i = 0;

		// Dateizeiger auf Zeilenanfang
		fseek(fp, -(long)(strlen(line) + 1), SEEK_CUR);

		pos++;
		for (; isspace(*pos); pos++); // Leerzeichen uebergehen

		*pos++ = '"'; // quote einfuegen

		for (; IsPrint(value[i]); pos++, i++) {
			
			*pos = value[i]; // Zeichenkette aufnehmen
		}

		*pos = 0;
		// unquote einfuegen
		strcat(pos++, "\"");
		// linebreak
		strcat(pos, "\n");

		//fprintf(fp, line);
		//fflush(fp);
		fputs(line, fp);

		return 1;
	}

	return 0;
}

// Lesen eines vorzeichenlosen Integerwertes
unsigned int CDFFileIni::GetUInt(char *section, char *key, int defaultVal, char *fileName)
{
	char sectionName[MAX_SECTION_NAME];
	char line[0xFF];
	FILE *fp;
	unsigned int res = (defaultVal < 0) ? 0 : defaultVal;

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
	
		return res;
	}
	if (setvbuf(fp, NULL, _IONBF, 0)) {
		
		fclose(fp);
		return res;
	}

	sprintf(sectionName, "[%s]", section);
	if (FindSection(fp, sectionName, 0)) {
		if (FindKey(fp, key, 0, line, 0xFF)) {
			res = GetNum(fp, defaultVal, line);
		}
	}

	fclose(fp);

	return res;
}

// Lesen einer Zeichenkette
int CDFFileIni::GetString(char *section, char *key, char *value, char *fileName)
{
	char sectionName[MAX_SECTION_NAME];
	char line[0xFF];
	FILE *fp;
	int res = 0;

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
	
		return res;
	}
	if (setvbuf(fp, NULL, _IONBF, 0)) {
		
		fclose(fp);
		return res;
	}

	sprintf(sectionName, "[%s]", section);
	if (FindSection(fp, sectionName, 0)) {
		if (FindKey(fp, key, 0, line, 0xFF)) {
			res = GetStr(fp, value, line);
		}
	}

	fclose(fp);

	return res;
}

// Schreiben einer Zeichenkette
int CDFFileIni::SetString(char *section, char *key, char *value, char *fileName)
{
	char sectionName[MAX_SECTION_NAME];
	char line[0xFF];
	FILE *fp;
	int res = 0;

	if ((fp = fopen(fileName, "r+")) == (FILE*)0) {
	
		return res;
	}
	/*
	if (setvbuf(fp, NULL, _IONBF, 0)) {
		
		fclose(fp);
		return res;
	}
	*/
	sprintf(sectionName, "[%s]", section);
	if (FindSection(fp, sectionName, FF_CREATE)) {
		if (FindKey(fp, key, FF_CREATE, line, 0xFF)) {
			res = SetStr(fp, value, line);
		}
	}

	fclose(fp);

	return res;
}


