/**
 * @file  DFLog.h
 * @brief Schnittstelle fuer die Klasse CDFLog.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFLOG_H_
#define _DFLOG_H_

#include <time.h>

// Initialisierungsdatei verwalten
#include "DFFileIni.h"
// Hilfsklasse zur Verwaltung alter Logdateien
#include "DFLogCheck.h"

/* Die Logklasse ist fuer ein standardmaessiges logen von Funktionsaufrufen,
 * so dass Fehler in der Verarbeitung auch noch nach einigen Tagen 
 * nachfolzogen werden koennten.
 * Sie ist ueber eine Initialisierungsdatei mit grundinformationen versorgbar.
 *
 */
#define SUB_FOLDER           "TBComLog"

#define INI_FILENAME         "TBCom.ini"
// YYYY_MM_DD_DFComDLL_channelID.log
#define LOG_FILENAME         "%s%s20%02i_%02i_%02i_TBCom_%i.log"
#define LOG_SEARCH_FILENAME  "%s%s20??_??_??_TBCom_*.log"
#define LOG_DELETE_FILENAME  "%s%s%s"

#ifdef _WIN32
#define PATH_BASE            ".\\"
#else
#define PATH_BASE            "./"
#endif

#ifdef _WIN32
#define fullpath             _fullpath
#define mkdir                _mkdir
#define stat                 _stat
#define finddata_t           _finddata_t
#define findfirst            _findfirst
#define findnext             _findnext
#define findclose            _findclose
#endif

class CDFLog
{
	bool m_bEnable;                  // Log aktiv?
	char m_szFileName[260];          // Dateiname der Logdatei
	bool m_bEraseFiles;              // Alte Dateien loeschen
	bool m_bElapse;                  // Ausfuehrungszeit angeben
	int m_bWriteList;               // Listendaten mitloggen
public:
	bool m_bTrace;                   // Tracing aktivieren
	char m_szFileNameTrace[260];     // Dateiname der Tracedatei

public:
	CDFLog(short channelID);
	virtual ~CDFLog();

	// Erzeugen einer Pfadangabe
	void MakeFilePath(char *path_buffer, char *szFileName, char *szFileExt);

	// Einlesen der Initialisierungsdatei
	void logIniRead();
	// Pruefen auf alte Dateien.
	void logCheck();

	// Schreiben einer Logzeile, wenn Log aktiv.
	void logBegin(const char *Format, ...); // Mit Uhrzeit
	void logAppend(const char *szFormat, ...);
	void logEnd(const char *szFormat, ...); // Mit Zeilenumbruch
	char *logGetBasePath(char *basePath)
	{
		sprintf(basePath, "%s%s", m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder);
		return basePath;
	}
	int logIsWriteList() { return m_bEnable ? m_bWriteList : 0; }
protected:
	// Zugewiesene Kanalnummer
	short m_channelID;
	// Zu verwendende Initialisierungsdatei
	CDFFileIni m_logIni;
	// Start des Logvorganges
	clock_t m_clockBegin;
	// Alte Logdateien verwalten
	CDFLogCheck m_logCheck;
};

#endif

