#ifndef SETUP_H

#define SETUP_H

//#ifdef PC_SIMULATION
#pragma pack (1)
//#endif 

#define EEP_SIZE	0x1000		/* 4kByte EEPROM */
#define EEP_SIZE_SU 0x200		/* 512kByte fr Setup-Daten */
#define EEP_SIZE_PT 0x84		/* Bereich fr Pointer */
#define FLASH_BAS	0xFC000		/* Startadresse im Flash fr Setup-Daten */
#define FLASH_SIZE_SU 0x4000	/* Ab Adresse 0xFC000 16kByte fr Setup-Daten */
#define FLASH_SIZE_SU_EXT 0x8000 // Erweiterter Konfigbereich (nur bei TB3)
#define FLASH_SIZE_SU_V2  (FLASH_SIZE_SU+FLASH_SIZE_SU_EXT)	// Gesammtkonfig gre ab Hardware V2
#define FLASH_KONFIG_START	0xFC000 /* Startadresse fr Konfig-Daten im Flash */
#define FLASH_LIST_START	0xE0000	/* Abhngig von Setup-Var. "Speicher"  Speicher = 0 */
#define FLASH_LIST_START_1	0xC0000 /* Speicher = 1 */
#define FLASH_LIST_START_2	0xA0000	/* Speicher = 2 ... */
#define FLASH_LIST_START_3	0x80000
#define FLASH_LIST_START_4	0x60000
#define FLASH_LIST_START_5	0x40000
#define FLASH_LIST_START_6	0x20000
#define FLASH_SIZE_LIST		0x18000	/* Speicher = 0 */
#define FLASH_SIZE_LIST_1	0x38000 /* Speicher = 1 ... */
#define FLASH_SIZE_LIST_2	0x58000
#define FLASH_SIZE_LIST_3	0x78000
#define FLASH_SIZE_LIST_4	0x98000
#define FLASH_SIZE_LIST_5	0xB8000
#define FLASH_SIZE_LIST_6	0xD8000
#define DATEN_SIZE  0xE0000	/* max. Speicher fr Daten */
/* Pointer fr Datenspeicherung */
//	Pointer im EEPROM 
//		Adresse des Pointers der im EEPROM liegt und aud das Flash zeigt
#define P_DAT_STRUCT2	0x0000	/* Datensatzbeschreibung */
#define P_KONFIG		0x0004	/* Konfiguration */
#define P_START_RING	0x0008	/* Startadresse Ringspeicher */
#define P_END_RING		0x000C	/* Endadresse Ringspeicher */
#define P_READ_RING		0x0010	/* Readpointer Ringsp. */
#define P_WRITE_RING	0x0014	/* Writepointer Ringspeicher */
#define P_WRITE_R_1		0x0018	/* Writepointer R. letzte Schreibperation (bezogen auf Datensatz) */
#define P_WRITE_R_2		0x001C	/* Writepointer R. vor letzte Schreibperation (bezogen auf Datensatz) */
#define P_WRITE_R_3		0x0020	/* Writepointer R. vor vor letzte Schreibperation (bezogen auf Datensatz) */
#define P_LIST_STRUCT	0x0024	/* Startadresse Listenbeschreibung */
#define P_LIST_0		0x0028	/* Adresse der Daten der Liste Nr. 0 im Flash */
								/* Die weiteren Adressen sind nachfolgend gespeichert */
								/* z.B. Adresse von Liste Nr. 4  P_LIST_0+4*sizeof(ULONG) */
#define P_LIST_F		0x002C	/* Adresse der festen Listen */
#define P_LIST_ART		0x0030	/* 4 Byte (long) mit Bitfeld LSB -> Liste 0; Bit 1 feste L.*/
/* #define P_LIST_19     0x0074*/  /* Pointer auf letzte Liste */
#define P_DAT_STRUCT	0x0080	/* Kopie von P_DAT_STRUCT Fehler von ATMega zu umgehen */
#define PASSWORT		0x0084	/* Passwort fr Kommunikation max. 16 Zeichen + 0 */
// PASSWORT Ende        0x0095
#define P_SETUP_VERSION	0x00A0	// wird nur vom Setup-Programm ausgewertet

//  Daten im EEPROM
/* Bereich im EEPROM der nicht vom Setup-Programm berschrieben wird */
#define LCD_KONTRAST	0x0200	/* Kontrastwert LCD */
#define SERIEN_NUMMER	0x0202	/* 2 Byte */
#define SERIEN_NUMMER2	0x02D2	/* Hherwertige 2 Byte der Seriennummer ab V3.2h */
#define UHR_KORREKTUR	0x0204	/* Korrekturwert fr RTC um Ganggenauigkeit zu optimieren */
#define ALPHA_TIMER_VAL	0x0206	/* Wartezeit fr Eingabe von Buchstaben ber 10ner-Tastatur */
#define EING_ART_BUCHST 0x0208	/* Eingabeart Buchstaben TRUE -> Mehrfachd. mit TO */
								/*						FALSE -> ber F-Tasten */

/* Datensatz im Flash incl. Verzeigerung und Crc */
/*	Aufbau:	*/
/*		Byte 0:		Lnge des Datensatzes einschlieslich CRC (x+11) */
/*		Byte 1..4:	Pointer auf nchsten Datensatz bzw. freien Speicher */
/*		Byte 5..8:	Pointer auf vorhergehenden Datensatz (NULL -> kein Vorgnger) */
/*		Byte 9..9+(x-1):x-Byte Daten */
/*		Byte 9+x,10+x: CRC nach Modbus-Norm */

// Offset
#define DS_LEN	0	/* Lnge Datensatz */
#define DS_NP	1	/* nchster Datensatz */
#define DS_PP	5	/* vorhergehender Datensatz */
#define DS_DAT	9
#define DS_CRC_X 10	/* Offset CRC - x */
#define DS_KOPF	9
#define DS_KOPF_CRC	11	/* Anzahl Byte fr Datensatz ohne Nutzdaten */

/* max. Lnge Datensatz ohne Verwaltungsdaten */
#define MAX_DATENSATZ (255-DS_KOPF_CRC)
#define MAX_FELD	20	/* max. Felder in einem Datensatz */
#define MAX_AN_FELD	21	/* 20 Zeichen + 0 */

#define MAX_UNTERPUNKTE 100	/* max. Unterpunkte im Baum (Konfig, Datensatzb. und Listenb.) */

#define MAX_DATENSAETZE	70577	/* max. Anzahl Datenstze im Timeboy */

/* Datensatzbeschreibung (nur Nutzdaten ohne Verwaltungsinformationen) */
/*	P_DAT_STRUCT im EEPROM zeigt auf den Begin der Datensatzbeschreibung im Flash */
/*  Ab dem Begin ist ein Array von max. "MAX_DATENSATZ_BESCHREIBUNG" Zeigern fr jede Datensatzbeschreibung */
/*	Die Pos. des Zeigers (ab 0) ist die Nummer der Datensatzbeschr. */
/*	Die Zeiger zeigen auf je eine Structur "TDatensatzBeschreibung" */
#define MAX_DATENSATZ_BESCHREIBUNG	20	/* auch max. Listen */
#define MAX_DS_NAME		17	/* max. Zeichen fr Datensatz/Feld-Name incl. abschliesende Null */

typedef struct StructDatensatzBeschreibung {
	UCHAR Felder;	/* Anzahl Felder im Datensatz */
	UCHAR Name[MAX_DS_NAME]; /* Name der Datensatzbeschreibung */
	ULONG P;	/* Pointer auf Feldbeschreibungen (Typ "TDatensatzFeld") */
} TDatensatzBeschreibung;

typedef struct StructDatensatzFeld {
	UCHAR Typ;		/* Aufzhlung Typ des Feldes */
	UCHAR Cnt;		/* Lnge des Feldes inc. anbschlieende '\0' bei Strings */
	UCHAR Name[MAX_DS_NAME];	/* Name des Feldes */
} TDatensatzFeld;

//	Feldtypen
#define TF_ULONG	1	/* 4 Byte */
#define TF_DT		2	/* Date und Time 7 Byte -> 4 Byte Date und 3 Byte Time */
#define TF_NUM		3	/* wie Typ TF_A_NUM, es sind jedoch nur Ziffern erlaubt */
#define TF_A_NUM	4	/* Alpha-Numerisch ASCII Lngenangabe notwendig */
#define TF_DATE		5	/* 4 Byte Y100,Y,M,D */
#define TF_TIME		6	/* 3 Byte H,Mi,S */
/*  Lnge der Feldtypen */
#define TF_ULONG_LEN	4	/* 4 Byte */
#define TF_DATE_LEN		4	/* 4 Byte Y100,Y,M,D */
#define TF_TIME_LEN		3	/* 3 Byte H,Mi,S */
#define TF_DT_LEN		7	/* Date und Time 7 Byte -> 4 Byte Date und 3 Byte Time */

/* Allgemeine Def. fr Konfiguration */
#define MAX_MENUE_TXT	17	/* Lnge Texte im Menue incl. NULL */
#define MAX_S_FELD_TXT  17	/* Lnge fr Text S-Felder incl. NULL */
#define MAX_DATEN_LEN	41	/* max. Lnge fr ein Datensatzfeld */
#define MAX_BARCODE_LEN	61	/* max. Lnge fr Barcodeeingaben */

/* Konfiguration */
/*	Bediensteuerung */
/*	Die Bedienung wird durch die Konfig-Structur gesteuert. */
/*	Die Konfig-S. ist durch Pointer strukturiert. */
/*  Die Bedienung arbeitet diese Srtructur ab, wie ein Interpreter. */
/*	"P_KONFIG" zeigt auf den ersten Eintrag der Verketteten Structuren.(wird meist nach "On" ausgefhrt) */

/*		Typen der Bedienung */
#define TB_MENUE		1
#define TB_DATENSATZ	2
#define TB_SYSTEM		3	/* Sytemmenue */
#define TB_KONFIG_ROOT	4	
#define TB_LISTE		5	/* Liste oder Daten anzeigen (nicht speichern) */
#define TB_CLR_LAST		6	/* Letzten Datensatz lschen */
#define TB_STAND_BY		7	/* Bereithalten fr Kommunikation mit Dockingstation */
#define TB_GLOB_VAR		8	/* Globale Variable anzeigen */
/*		Arten der Menueauswahl */
#define TBMA_NORM		0x01	/* Auswahl ber Pfeil- und F-Tasten */
#define TBMA_BARCODE	0x02	/* Auswahl ber Barcode EK1->Eintrag 1; EK2->Eintrag 2 ... */
/* Einlesemetoden fr Datensaetze */
#define TBDA_NORM	0		/* Standard */
#define TBDA_LIST	1		/* Schon eingegebene Daten werde in Listenform angezeigt */
/* Einlesemetoden fr Felder */
#define TBFE_NORM	1
#define TBFE_GET_TIME		2	/* aktuelle Datum Uhrzeit aud Timeboy */
#define TBFE_LIST_AUSWAHL	3	/* aus Liste auswhlen */
#define TBFE_KONSTANTE		4	/* Konstante in den Datensatz schreiben */
#define TBFE_BESTAETIGUNG	5	/* Besttigung abfragen (fllt kein Datenfeld) */
#define TBFE_GLOB_VAR		6	/* aus Glog.-Var. kopieren */
#define TBFE_SERIENNUMMER	7	/* Seriennummer des TB bernehmen */
/* Quellen fr Eingabe von Daten (Maske in Var. Flag) */
#define NFF_TASTATUR	0x80
#define NFF_BARCODE		0x40
#define NFF_NO_RET		0x01	/* kein CR bei Barcodeeingabe notwendig */
#define NFF_ANZEIGEN	0x0E	/* 3 Bit 8 Werte fr Anzeigedauer wenn kein Return notwendig */
/* Verhalten am Ende einer Eingabekette oder bei ESC */
#define EV_MENUE	0	/* zurck ins Menue */
#define EV_AUS		1	/* Timeboy ausschalten */
#define EV_OFFSET_ESC 1 /* Offset fr Feldnummer bei ESC */
#define EV_OFFSET	2	/* Offset fr Feldnummer Ende Eingabekette */
/* Anzeigemethode fr Listendaten bzw eingegebene Datenstze */
#define LAM_NORMAL	0	/* immer vier Datenstze auf einmahl */
#define LAM_DATENSATZ	1 /* immer ein Datensatz im Display; Felder untereinander; */
						// Pfeil r/l Felder r/l bei langen Feldern;
						// Pfeil h/r weitere Felder (bei mehr als 4); F1/F2 Datensatz vor/zurck
/* Ein Varhalten */
#define EVERH_AKTUEL 0	/* Nach Ein auf der aktuellen Pos vor Aus bleiben */
#define EVERH_MENUE	1	/* Nach Einschalten zurck in das Menue */
						/* >1 -> Eintrag aus Hauptmenue starten */
/* Flags Datensatz Einlesen */
#define BDEF_W_UEBER	0x01	/* Bei Schleifen bereits eingegebene Werte als */
								/* Vorgabe fr Eingabe bernehmen */
#define BDEF_NICHT_SPEICHERN 0x02 /* Datensatz nicht speichern (nur Globale Variable) */
#define BDEF_ZUSATZ_RETURN	0x04	/* Abfrage "Speichern ?" am Ende der Eingabekette */
#define BDEF_RUECKSPRUNG_MENUE 0x08	/* Nach Einlesen eines Datensatzes Rcksprungmen anzeigen */
/* Flags Barcode fr TBExListAuswahl.SelBarcode */
#define BDLB_SEL_BARCODE	0x01	/* Selection in Liste mit Barcode freigeben */
#define BDLB_BESTAETIGEN	0x02	/* Ausgewhlten Eintrag besttigen mit Ret */
#define BDLB_SEL_STRING		0x04	/* Selection in Liste nach eingegebener Zeichenfolge freigeben */
/* Flags Allgemein (Konfig Root) */
#define KRF_AKTIV_OFF		0x01	/* auch bei ausgeschaltetem Gert IR-Kommunikatio zulassen */
#define KRF_NO_SOMMERZEIT	0x02	/* Sommerzeit nicht automatisch umschalten */

#ifdef AE3
typedef struct StructBedien {
	volatile UCHAR	Typ;	/* Typ der Bedienung Siehe TB_..... */
	volatile UCHAR  d1,d2;	/* Reserve fr Erweiterung */
	volatile ULONG	P;		/* Pointer auf Structur welche durch Typ" ausgewhlt wird */
} TBedien;
#else
volatile typedef struct StructBedien {
	volatile UCHAR	Typ;	/* Typ der Bedienung Siehe TB_..... */
	volatile UCHAR  d1,d2;	/* Reserve fr Erweiterung */
	volatile ULONG	P;		/* Pointer auf Structur welche durch Typ" ausgewhlt wird */
} TBedien;
#endif

typedef struct StructListAnz {
	UCHAR	ListNum;	/* Nummer der Liste oder Datensatzbeschreibung die angezeigt wird */
	UCHAR	Name[MAX_MENUE_TXT];
	UCHAR	Methode;	// Darstellmethode (Siehe LAM_...)
	UCHAR	Daten;		// 1 -> Daten Anzeigen
	UCHAR   Dummy[6];	// Reserve
} TBListAnz;

typedef struct StructMenue {
	UCHAR	AuswahlTyp;	/* Art der Auswahl der Menueeintrge */
	UCHAR	EintragCnt;	/* Anzahl Eintrge im Menue */
	UCHAR	Name1[MAX_MENUE_TXT];		/* berschrift1 des Menues */
	UCHAR	Name2[MAX_MENUE_TXT];		/* berschrift2 des Menues */
	UCHAR	K1Txt;		/* Text in Feld K1 Bit 0...3 Nummer-Globale-Variable */
						/*                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. */
	UCHAR	K2Txt;		/* Text im Feld K2 */
	UCHAR   Dummy[6];	// Reserve
	ULONG	P;			/* Pointer auf Liste der Eintrge */
} TBMenue;

typedef struct StructKonfigRoot {
#ifdef PC_SIMULATION
	unsigned short	AutoAus;	/* Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfogt */
#else
	UINT	AutoAus;	/* Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfogt */
						/* 0 -> kein Automatische abschaltung */
#endif
	UCHAR	EintragCnt;	/* Anzahl Eintrge in KonfigRoot */
	UCHAR	EinVerhalten;	/* Verhalten nach Einschalten */
	UCHAR	Flags;		/* diverse Flags Sihe KRF_... */
	UCHAR	Dummy[7];	/* Reserve */
	ULONG	P;			/* Pointer auf Pointerarry mit "EintragCnt" Eintrgen */
} TBKonfigRoot;

typedef struct StructMenueEintrag {
	UCHAR	Name[MAX_MENUE_TXT];	/* Menue Text */
	UCHAR	EintragCnt;	/* Anzahl Eintrge in KonfigRoot */
	UCHAR	PassL;		/* Passwort Low-Byte */
	UCHAR	PassH;		/* Passwort High-Byte  0 -> kein Passwort */
	UCHAR	Dummy[6];	/* Reserve */
	ULONG	P;			/* Pointer auf Aktion nach Auswahl des Menuepunktes */
} TBMenueEintrag;

typedef struct StructDatensatzEinlesen {
	UCHAR	DsNum;		/* Nummer der Datensatzbeschreibung (ab 0) */
	UCHAR	Felder;		/* Anzahl Felder (mu mit Datensatzbeschreibung bereinstimmen) */
	UCHAR	CrV;		/* Verhalten bei Enter-Taste */
	UCHAR	Name[MAX_MENUE_TXT];		/* berschrift */
	UCHAR	Typ;		/* Bedien-Typ des Einlesens */
	UCHAR	Flags;		/* Diverse Flags Definiert als BDEF_.... */
	UCHAR	K1Txt;		/* Text in Feld K2 Bit 0...3 Nummer-Globale-Variable */
						/*                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. */
	UCHAR	Dummy[6];	/* Reserve */
	ULONG	P;			/* Pointer auf Liste der Structuren fr Einlesen der Felder */
						/*(Anzahl und Reihenfolge mu mit Datensatzbeschreibung bereinstimmen) */
} TBDatensatzEinlesen;

typedef struct StructExBestaetigung {
	UCHAR	Text1[MAX_MENUE_TXT];		/* Hinweistext */
	UCHAR	Text2[MAX_MENUE_TXT];		/* Hinweistext */
	UCHAR	Text3[MAX_MENUE_TXT];		/* Hinweistext */
} TBExBestaetigung;

typedef struct StructExNumFeld {
	UCHAR	Flags;	/* Flags z.B. Einlesen ber Barcode erlaubt */
	ULONG	Min;
	ULONG	Max;
	ULONG	Konstante;
	UCHAR	KonstAscii[MAX_AN_FELD];
} TBExNumFeld;

typedef struct StructExListAuswahl {
	UCHAR	Sel;	/* Flag Listenauswahl mit Selection */
	UCHAR	SelDFeld;	/* Feld im Datensatz fr Selection (nur wenn "Sel" = TRUE) */
	UCHAR	SelLFeld;	/* Feld in der Liste nach dem selectiet wird (nur wenn "Sel" = TRUE) */
	UCHAR	List;	/* Nummer der Liste */
	UCHAR	AuswahlFeld;	/* Feld in der Liste nach dem der Benutzer auswhlt */
	UCHAR	DatFeld;	/* Feld in der Liste das im Datensatz gespeichert wird */
	UCHAR	SelBarcode;
} TBExListAuswahl;

typedef struct StructFeldEinlesen {
	UCHAR	Typ;		/* Typ der Einlesemetode "TBFE..." */
	UCHAR	DFNum;		/* Nummer des Datensatzfeldes (ab 0) */
	UCHAR	EscV;		/* Verhalten bei ESC-Taste */
	UCHAR	Name[MAX_MENUE_TXT];		/* berschrift */
	UCHAR	K2Txt;		/* Text in Feld K2 Bit 0...3 Nummer-Globale-Variable */
						/*                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. */
	UCHAR	GlobVar;	/* Feld-Wert in Globale Var. kopieren (Aufbau wie K2Txt) */
	UCHAR	BarcodeTyp;	/* nur bestimmten Typ bei Eingabe zulassen */
	UCHAR	BarcodeVon;	/* Bereich aus Barcode ausschneiden 1 -> erstes Zeichen 0 -> deaktiviert */
	UCHAR	BarcodeBis;	
	UCHAR	Dummy[3];	/* Reserve */
	union Ex {
		TBExBestaetigung	Bestaetigung;
		TBExNumFeld			NumFeld;
		TBExListAuswahl		ListAuswahl;
	} Ex;
} TBFeldEinlesen;

typedef struct StructSetupAllgemein {
	UCHAR	SFeld[4][2][MAX_S_FELD_TXT];
	ULONG	P;			/* Pointer auf Root StructBedien */
} TBAllgemein;

// Bitfeld fr Systemmen
#define SM_DATUM_UHR		0x01
#define SM_LCD_KONTRAST		0x02
#define SM_SCANNER_TEST		0x04
#define SM_AKKU_SPANNUNG	0x08
#define SM_UHR_KORREKTUR	0x10
#define SM_TO_EINGABE		0x20
#define SM_EINGABE_ART		0x40

#pragma  pack()
#endif
